# -*- coding: utf-8 -*-

# Copyright (c) 2008/2009 Andrey Vlasovskikh
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

r'''A DOT language parser using funcparserlib.

The parser is based on [the DOT grammar][1]. It is pretty complete with a few
not supported things:

* Ports and compass points
* XML identifiers

At the moment, the parser builds only a parse tree, not an abstract syntax tree
(AST) or an API for dealing with DOT.

  [1]: http://www.graphviz.org/doc/info/lang.html
'''

import os
import sys
import codecs
from re import MULTILINE, DOTALL
from pprint import pformat
from funcparserlib.util import pretty_tree
from funcparserlib.lexer import make_tokenizer, Token, LexerError
from funcparserlib.parser import (some, a, maybe, many, finished, skip,
    oneplus, forward_decl, NoParseError)

from blockdiag.utils.namedtuple import namedtuple

ENCODING = 'utf-8'

Graph = namedtuple('Graph', 'type id stmts')
Network = namedtuple('Network', 'id stmts')
SubGraph = namedtuple('SubGraph', 'id stmts')
Node = namedtuple('Node', 'id attrs')
Attr = namedtuple('Attr', 'name value')
Edge = namedtuple('Edge', 'nodes attrs')
DefAttrs = namedtuple('DefAttrs', 'object attrs')


class ParseException(Exception):
    pass


def tokenize(str):
    'str -> Sequence(Token)'
    specs = [
        ('Comment', (r'/\*(.|[\r\n])*?\*/', MULTILINE)),
        ('Comment', (r'//.*',)),
        ('NL',      (r'[\r\n]+',)),
        ('Space',   (r'[ \t\r\n]+',)),
        ('Name',    (ur'[A-Za-z_\u0080-\uffff][A-Za-z_.0-9\u0080-\uffff]*',)),
        ('Op',      (r'[{};,=\[\]]',)),
        ('Number',  (r'-?(\.[0-9]+)|([0-9]+(\.[0-9]*)?)',)),
        ('String',  (r'(?P<quote>"|\').*?(?<!\\)(?P=quote)', DOTALL)),
    ]
    useless = ['Comment', 'NL', 'Space']
    t = make_tokenizer(specs)
    return [x for x in t(str) if x.type not in useless]


def parse(seq):
    'Sequence(Token) -> object'
    unarg = lambda f: lambda args: f(*args)
    tokval = lambda x: x.value
    flatten = lambda list: sum(list, [])
    node_flatten = lambda l: sum([[l[0]]] + list(l[1:]), [])
    n = lambda s: a(Token('Name', s)) >> tokval
    op = lambda s: a(Token('Op', s)) >> tokval
    op_ = lambda s: skip(op(s))
    id = some(lambda t:
        t.type in ['Name', 'Number', 'String']).named('id') >> tokval
    make_graph_attr = lambda args: DefAttrs(u'graph', [Attr(*args)])
    make_edge = lambda x, x2, xs, attrs: Edge([x, x2] + xs, attrs)

    node_id = id  # + maybe(port)
    node_list = (
        node_id +
        many(op_(',') + node_id)
        >> node_flatten)
    a_list = (
        id +
        maybe(op_('=') + id) +
        skip(maybe(op(',')))
        >> unarg(Attr))
    attr_list = (
        many(op_('[') + many(a_list) + op_(']'))
        >> flatten)
    graph_attr = id + op_('=') + id >> make_graph_attr
    node_stmt = node_id + attr_list >> unarg(Node)
    # We use a forward_decl becaue of circular definitions like (stmt_list ->
    # stmt -> subgraph -> stmt_list)
    group_stmt = (
        graph_attr
        | node_stmt
    )
    group_stmt_list = many(group_stmt + skip(maybe(op(';'))))
    group = (
        skip(n('group')) +
        maybe(id) +
        op_('{') +
        group_stmt_list +
        op_('}')
        >> unarg(SubGraph))
    network_stmt = (
        graph_attr
        | group
        | node_stmt
    )
    network_stmt_list = many(network_stmt + skip(maybe(op(';'))))
    network = (
        skip(n('network')) +
        maybe(id) +
        op_('{') +
        network_stmt_list +
        op_('}')
        >> unarg(Network))
    stmt = (
        network
        | group
        | graph_attr
    )
    stmt_list = many(stmt + skip(maybe(op(';'))))
    graph = (
        maybe(n('diagram')) +
        maybe(id) +
        op_('{') +
        stmt_list +
        op_('}')
        >> unarg(Graph))
    dotfile = graph + skip(finished)

    return dotfile.parse(seq)


def parse_string(string):
    try:
        return parse(tokenize(string))
    except LexerError, e:
        message = "Got unexpected token at line %d column %d" % e.place
        raise ParseException(message)
    except Exception, e:
        raise ParseException(str(e))


def parse_file(path):
    input = codecs.open(path, 'r', 'utf-8').read()
    return parse_string(input)



# -*- coding: utf-8 -*-
#  Copyright 2011 Takeshi KOMIYA
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import sys
import math
import blockdiag.DiagramDraw
from blockdiag.utils.XY import XY
from blockdiag import noderenderer
from blockdiag.DiagramMetrix import DiagramMetrix


class DiagramDraw(blockdiag.DiagramDraw.DiagramDraw):
    @property
    def groups(self):
        return self.diagram.groups

    def pagesize(self, scaled=False):
        # FIXME: force int'ize
        xy = super(DiagramDraw, self).pagesize(scaled)
        return XY(int(xy.x), int(xy.y))

    def _draw_background(self):
        super(DiagramDraw, self)._draw_background()

        self._draw_trunklines()

    def _draw_trunklines(self):
        metrix = self.metrix.originalMetrix()
        for network in self.diagram.networks:
            m = metrix.network(network)
            self.drawer.line(m.trunkline, fill=self.fill)

            # FIXME: first network links to global network
            if network == self.diagram.networks[0]:
                pt1 = m.top()
                pt0 = XY(pt1.x, pt1.y - m.metrix.spanHeight * 2 / 3)

                self.drawer.line([pt0, pt1], fill=self.fill)

    def draw(self):
        super(DiagramDraw, self).draw()

        self._draw_trunkline_labels()

    def _draw_trunkline_labels(self):
        for network in self.diagram.networks:
            if network.display_label:
                m = self.metrix.network(network)
                self.drawer.textarea(m.textbox, network.display_label,
                                     fill=self.fill, halign="right",
                                     font=self.font,
                                     fontsize=self.metrix.fontSize)

    def node(self, node, **kwargs):
        m = self.metrix

        for connector in m.node(node).connectors:
            for i in range(node.xy.y + 1, connector.network.xy.y):
                networks = (n for n in self.diagram.networks if n.xy.y == i)
                for nw in networks:
                    if nw.xy.x <= node.xy.x <= nw.xy.x + nw.width - 1:
                        connector.jumps.append(nw)

            self.draw_connector(connector)

            if connector.network in node.address:
                label = node.address[connector.network]
                self.drawer.textarea(connector.textbox, label, fill=self.fill,
                                     halign="left", font=self.font,
                                     fontsize=self.metrix.fontSize)

        super(DiagramDraw, self).node(node, **kwargs)

    def draw_connector(self, connector):
        m = self.metrix

        if connector.jumps:
            pt1, pt2 = connector.line
            lines = [pt1]
            for nw in connector.jumps:
                pt = m.cell(nw).top().y - m.spanHeight / 2
                r = m.cellSize / 2

                lines.append(XY(pt1.x, pt - r))
                lines.append(XY(pt1.x, pt + r))

                box = [pt1.x - r, pt - r, pt1.x + r, pt + r]
                self.drawer.arc(box, 270, 90, fill=self.fill)

            lines.append(pt2)

            for j, point in enumerate(lines[::2]):
                self.drawer.line([point, lines[j * 2 + 1]], fill=self.fill)
        else:
            self.drawer.line(connector.line, fill=self.fill)


from DiagramMetrix import DiagramMetrix
DiagramDraw.set_metrix_class(DiagramMetrix)



# -*- coding: utf-8 -*-

import os
import pep8

CURRENT_DIR = os.path.dirname(os.path.abspath(__file__))
BASE_DIR = os.path.dirname(CURRENT_DIR)


def test_pep8():
    arglist = [
        '--statistics',
        '--filename=*.py',
        '--show-source',
        '--repeat',
        '--exclude=SVGdraw.py',
        #'--show-pep8',
        #'-qq',
        #'-v',
        BASE_DIR,
    ]

    options, args = pep8.process_options(arglist)
    runner = pep8.input_file

    for path in args:
        if os.path.isdir(path):
            pep8.input_dir(path, runner=runner)
        elif not pep8.excluded(path):
            options.counters['files'] += 1
            runner(path)

    pep8.print_statistics()
    errors = pep8.get_count('E')
    warnings = pep8.get_count('W')
    message = 'pep8: %d errors / %d warnings' % (errors, warnings)
    print message
    assert errors + warnings == 0, message



# -*- coding: utf-8 -*-

import tempfile
from nwdiag.builder import *
from nwdiag.elements import *
from nwdiag.diagparser import *


def __build_diagram(filename):
    import os
    testdir = os.path.dirname(__file__)
    pathname = "%s/diagrams/%s" % (testdir, filename)

    str = open(pathname).read()
    tree = parse(tokenize(str))
    return ScreenNodeBuilder.build(tree)


def test_diagram_attributes():
    screen = __build_diagram('diagram_attributes.diag')

    assert screen.node_width == 160
    assert screen.node_height == 160
    assert screen.span_width == 32
    assert screen.span_height == 32
    assert screen.fontsize == 16


def test_node_attributes():
    screen = __build_diagram('node_attributes.diag')

    network = screen.networks[0]
    assert screen.nodes[0].address[network] == '192.168.0.1'
    assert screen.nodes[1].address[network] == '192.168.0.2\n192.168.0.3'


def test_single_network_diagram():
    screen = __build_diagram('single_network.diag')

    assert len(screen.nodes) == 1
    assert len(screen.networks) == 1
    assert screen.nodes[0].label == 'A'
    assert screen.nodes[0].xy == (0, 0)


def test_two_networks_diagram():
    screen = __build_diagram('two_networks.diag')

    assert_pos = {'A': (0, 0), 'B': (0, 1)}
    for node in screen.nodes:
        print assert_pos[node.id], node.xy
        assert node.xy == assert_pos[node.id]


def test_connected_networks_diagram():
    screen = __build_diagram('connected_networks.diag')

    assert_pos = {'A': (0, 0), 'B': (1, 1)}
    for node in screen.nodes:
        print assert_pos[node.id], node.xy
        assert node.xy == assert_pos[node.id]


def test_group_inner_network_diagram():
    screen = __build_diagram('group_inner_network.diag')

    assert_pos = {'A': (0, 0), 'B': (1, 0)}
    for node in screen.nodes:
        print assert_pos[node.id], node.xy
        assert node.xy == assert_pos[node.id]

    assert len(screen.groups[0].nodes) == 2
    assert screen.groups[0].width == 2
    assert screen.groups[0].height == 1


def test_group_outer_network_diagram():
    screen = __build_diagram('group_outer_network.diag')

    assert_pos = {'A': (0, 0), 'B': (1, 0)}
    for node in screen.nodes:
        print assert_pos[node.id], node.xy
        assert node.xy == assert_pos[node.id]

    assert len(screen.groups[0].nodes) == 2
    assert screen.groups[0].width == 2
    assert screen.groups[0].height == 1


def test_group_across_network_diagram():
    screen = __build_diagram('group_across_network.diag')

    assert_pos = {'A': (0, 0), 'B': (1, 0), 'C': (2, 1)}
    for node in screen.nodes:
        print assert_pos[node.id], node.xy
        assert node.xy == assert_pos[node.id]

    assert len(screen.groups[0].nodes) == 3
    assert screen.groups[0].width == 3
    assert screen.groups[0].height == 2



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
import os, sys

sys.path.insert(0, 'src')
import nwdiag

long_description = \
        open(os.path.join("src","README.txt")).read() + \
        open(os.path.join("src","TODO.txt")).read()

classifiers = [
    "Development Status :: 3 - Alpha",
    "Intended Audience :: System Administrators",
    "License :: OSI Approved :: Apache Software License",
    "Programming Language :: Python",
    "Topic :: Software Development",
    "Topic :: Software Development :: Documentation",
    "Topic :: Text Processing :: Markup",
]

setup(
     name='nwdiag',
     version=nwdiag.__version__,
     description='nwdiag generate network-diagram image file from spec-text file.',
     long_description=long_description,
     classifiers=classifiers,
     keywords=['diagram','generator'],
     author='Takeshi Komiya',
     author_email='i.tkomiya at gmail.com',
     url='http://blockdiag.com/',
     license='Apache License 2.0',
     py_modules=['sphinxcontrib_nwdiag', 'nwdiag_sphinxhelper'],
     packages=find_packages('src'),
     package_dir={'': 'src'},
     package_data = {'': ['buildout.cfg']},
     include_package_data=True,
     install_requires=[
        'setuptools',
        'PIL',
        'funcparserlib',
        'blockdiag>=0.8.1',
         # -*- Extra requirements: -*-
     ],
     extras_require=dict(
         test=[
             'Nose',
             'minimock',
             'pep8',
         ],
         pdf=[
             'reportlab',
         ],
     ),
     test_suite='nose.collector',
     tests_require=['Nose','minimock','pep8'],
     entry_points="""
        [console_scripts]
        nwdiag = nwdiag.command:main
     """,
)

